package com.ywt.mg.services.enterprise;

import com.ywt.mg.configs.DomainConfigurer;
import com.ywt.mg.core.SqlHelper;
import com.ywt.mg.core.utils.Checker;
import com.ywt.mg.core.utils.FileUtil;
import com.ywt.mg.core.utils.ZipFileUtil;
import com.ywt.mg.core.utils.serializers.JsonSerializer;
import com.ywt.mg.domain.entities.*;
import com.ywt.mg.params.enterprise.marketting.BatchDownloadFilesRequest;
import com.ywt.mg.services.AuthService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class ApplyProjectService {

    private static final Logger logger = LoggerFactory.getLogger(ApplyProjectService.class);

    @Autowired
    private SqlHelper sqlHelper;
    @Autowired
    private AuthService authservice;
    @Autowired
    private MarketingApplyRepository marketingApplyRepository;
    @Autowired
    private UserRepository userRepository;
    @Autowired
    private MarketingImageRepository marketingImageRepository;
    @Autowired
    private MarketingJobTypeUserRepository marketingJobTypeUserRepository;
    @Autowired
    private DomainConfigurer domainConfigurer;
    @Autowired
    private MarketingProjectRepository marketingProjectRepository;

    public static final String APPLY_PROJECT_FILE_PATH = "./apply_project_file/files/";
    public static final String APPLY_PROJECT_ROOT_PATH = "./apply_project_file/";


    /**
     * 批量下载文件：
     * 1、先根据从前端选择的条件去数据库查找对应的记录
     * 2、遍历记录去下载对应的文件到服务器；
     * 3、将下载的文件打包；
     * 4、将打包的文件传给客户端
     * 5、删掉之前所有的文件；
     *
     * @param request
     * @return
     */
    public void batchDownloadFiles(BatchDownloadFilesRequest request, HttpServletResponse response) {
        try {
            // 1、先根据从前端选择的条件去数据库查找对应的记录
            Map<String, String> item = new HashMap<>();
            item.put("测试文档测试文档测试文档测试文档.doc", "https://ywt-files.oss-cn-shenzhen.aliyuncs.com//up/marketting_project/public/9-a30e19ed1c8d4b28ba83eb7a88a4f985.doc");
            item.put("下载.jpg", "https://ywt-files.oss-cn-shenzhen.aliyuncs.com//up/marketting_project/public/9-bb077b3512074ddaae0a12d797f20d02.jpg");
            item.put("测试文档.pdf", "https://ywt-files.oss-cn-shenzhen.aliyuncs.com//up/marketting_project/public/9-1b9a0a24f2be4cef882a22d1a5750ae5.pdf");

            // 2、遍历记录去下载对应的文件到服务器；
            List<String> fileUrl = new ArrayList<>();
            for (Map.Entry<String, String> entry : item.entrySet()) {
                logger.info(entry.getKey() + "--->" + entry.getValue());
                if (Checker.isNone(entry.getKey()) || Checker.isNone(entry.getValue())) {
                    continue;
                }
                String urlLink = entry.getValue();
                String pathName = APPLY_PROJECT_FILE_PATH + entry.getKey();
                // 下载单个文件
                downloadSingleFile(urlLink, pathName);
                fileUrl.add(pathName);
            }

            // 3、将下载的文件打包
            String zipFileName = "资料";
            if (!Checker.isNone(request.getUploadTimeStart()) && !Checker.isNone(request.getUploadTimeEnd())) {
                zipFileName += request.getUploadTimeStart().replace("-", "") + "~" + request.getUploadTimeEnd().replace("-", "");
            }
            zipFileName += ".zip";
            ZipFileUtil.stringFileUrlToZip(APPLY_PROJECT_ROOT_PATH + zipFileName, fileUrl);

            // 4、将打包的文件传给客户端
            downFile(response, APPLY_PROJECT_ROOT_PATH, zipFileName);
        } catch (Exception e) {
            logger.error("ApplyProjectService#batchDownloadFiles(request:{}){}", JsonSerializer.toJson(request), e.getMessage(), e);
        } finally {
            // 5、删掉之前所有的文件；
            FileUtil.deleteFolders(APPLY_PROJECT_ROOT_PATH);
        }
    }

    /**
     * 下载单个文件到服务器
     *
     * @param urlLink  oss的路径地址
     * @param pathName 本地文件（包含文件所在路径以及文件名称）
     */
    private static void downloadSingleFile(String urlLink, String pathName) {
        try {
            FileUtil.createFoldersIfNotExit(pathName);
            URL url = new URL(urlLink);
            //打开到此 URL 的连接并返回一个用于从该连接读入的 InputStream。
            InputStream inputStream = url.openStream();
            File file = new File(pathName);
            OutputStream outputStream = new FileOutputStream(file);
            int byteCount = 0;
            //1M逐个读取
            byte[] bytes = new byte[1024 * 1024];
            while ((byteCount = inputStream.read(bytes)) != -1) {
                outputStream.write(bytes, 0, byteCount);
            }
            inputStream.close();
            outputStream.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 将本地zip返回给前端
     *
     * @param response
     * @param FilePath
     * @param str
     */
    private void downFile(HttpServletResponse response, String FilePath, String str) {
        Map m = new HashMap();
        try {
            String path = FilePath + str;
            File file = new File(path);
            if (file.exists()) {
                InputStream ins = new FileInputStream(path);
                // 放到缓冲流里面
                BufferedInputStream bins = new BufferedInputStream(ins);
                // 获取文件输出IO流
                OutputStream outs = response.getOutputStream();
                BufferedOutputStream bouts = new BufferedOutputStream(outs);
                response.addHeader("content-disposition", "attachment;filename="+ new String(str.getBytes(), "iso-8859-1"));
                int bytesRead = 0;
                //1M逐个读取
                byte[] bytes = new byte[1024 * 1024];
                // 开始向网络传输文件流
                while ((bytesRead = bins.read(bytes)) != -1) {
                    bouts.write(bytes, 0, bytesRead);
                }
                // 这里一定要调用flush()方法
                bouts.flush();
                ins.close();
                bins.close();
                outs.close();
                bouts.close();
            }
        } catch (IOException e) {
            m.put("code", "-1");
            m.put("text", "附件下载出错：" + e.getMessage());
            e.printStackTrace();
        }
    }

}
